Package: ramwas
Type: Package
Title: Fast Methylome-Wide Association Study Pipeline for Enrichment
        Platforms
Version: 1.37.0
Date: 2019-02-18
Authors@R: c(person("Andrey A", "Shabalin",
                    role = c("aut", "cre"),
                    email = "andrey.shabalin@gmail.com",
                    comment = c(ORCID = "0000-0003-0309-6821")),
            person("Shaunna L", "Clark",
                    email = "slclark2@vcu.edu",
                    role = "aut"),
            person("Mohammad W", "Hattab",
                    email = "mwhattab@vcu.edu",
                    role = "aut"),
            person("Karolina A", "Aberg",
                    email = "kaaberg@vcu.edu",
                    role = "aut"),
            person("Edwin J C G", "van den Oord",
                    email = "ejvandenoord@vcu.edu",
                    role = "aut"))
Maintainer: Andrey A Shabalin <andrey.shabalin@gmail.com>
Description: A complete toolset for 
        methylome-wide association studies (MWAS).
        It is specifically designed for data from 
        enrichment based methylation assays,
        but can be applied to other data as well.
        The analysis pipeline includes seven steps:
        (1) scanning aligned reads from BAM files,
        (2) calculation of quality control measures,
        (3) creation of methylation score (coverage) matrix,
        (4) principal component analysis for capturing batch effects and
        detection of outliers,
        (5) association analysis with respect to phenotypes of interest
        while correcting for top PCs and known covariates,
        (6) annotation of significant findings, and
        (7) multi-marker analysis (methylation risk score) using elastic net.
        Additionally, RaMWAS include tools for joint analysis of methlyation
        and genotype data.
        This work is published in Bioinformatics,
        Shabalin et al. (2018) <doi:10.1093/bioinformatics/bty069>.
URL: https://bioconductor.org/packages/ramwas/
BugReports: https://github.com/andreyshabalin/ramwas/issues
License: LGPL-3
LazyLoad: yes
NeedsCompilation: yes
Depends: R (>= 3.3.0), methods, filematrix
VignetteBuilder: knitr
Suggests: knitr, rmarkdown, pander, BiocStyle,
        BSgenome.Ecoli.NCBI.20080805
Imports: graphics, stats, utils, digest, glmnet, KernSmooth, grDevices,
        GenomicAlignments, Rsamtools, parallel, biomaRt, Biostrings,
        BiocGenerics
biocViews: DNAMethylation, Sequencing, QualityControl, Coverage,
        Preprocessing, Normalization, BatchEffect, PrincipalComponent,
        DifferentialMethylation, Visualization
git_url: https://git.bioconductor.org/packages/ramwas
git_branch: devel
git_last_commit: df845e9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
Packaged: 2026-05-18 23:09:16 UTC; biocbuild
Author: Andrey A Shabalin [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0309-6821>),
  Shaunna L Clark [aut],
  Mohammad W Hattab [aut],
  Karolina A Aberg [aut],
  Edwin J C G van den Oord [aut]
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 05:04:40 UTC; unix
